==> verum-dezyne-2.18.1-x86_64-windows-debug.zip: dezyne-2.18.1/gnu/share/guile/site/3.0/ide/commands/simulate.scm <==
;;; Verum-Dezyne --- An IDE for Dezyne
;;;
;;; Copyright © 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
;;; Copyright © 2020 Rutger van Beusekom <rutger.van.beusekom@verum.com>
;;;
;;; This file is part of Verum-Dezyne.
;;;
;;; Verum-Dezyne is property of Verum Software Tools BV <support@verum.com>.
;;; All rights reserved.

;;
;;; Commentary:
;;;
;;; Code:

(define-module (ide commands simulate)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-26)
  #:use-module (srfi srfi-71)
  #:use-module (ice-9 getopt-long)
  #:use-module (ice-9 match)
  #:use-module (ice-9 rdelim)
  #:use-module (oop goops)
  #:use-module (dzn ast)
  #:use-module (dzn ast goops)
  #:use-module (dzn parse)
  #:use-module (ide shell-util)
  #:use-module (ide commander)
  #:use-module (ide commands daemon)
  #:use-module (ide config)
  #:use-module (ide daemon-config)
  #:use-module (ide pipe)
  #:use-module (ide util)
  #:use-module (ide command-line)
  #:export (sequence->error
            handle-simulate-result))

(define (parse-opts args)
  (let* ((option-spec
          '((help (single-char #\h))
            (import (single-char #\I) (value #t))
            (model (single-char #\m) (value #t))
            (no-compliance (single-char #\C))
            (no-deadlock (single-char #\D))
            (no-interface-livelock)
            (no-queue-full (single-char #\Q))
            (no-refusals (single-char #\R))
            (no-strict (single-char #\S))
            (queue-size (single-char #\q) (value #t))
            (queue-size-defer (value #t))
            (queue-size-external (value #t))
            (strict (single-char #\s))
            (trail (single-char #\t) (value #t))
            (trail-file (single-char #\T) (value #t))))
	 (options (getopt-long args option-spec))
	 (help? (option-ref options 'help #f))
	 (files (option-ref options '() '()))
         (usage? (and (not help?) (null? files))))
    (when (or help? usage?)
      (let ((port (if usage? (current-error-port) (current-output-port))))
        (format port "\
Usage: ide simulate [OPTION]... [FILE]...
Run simulate and make a trace view available.

  -C, --no-compliance    skip the compliance check
  -D, --no-deadlock      skip the deadlock check
  -Q, --no-queue-full    skip the external queue-full check
  -R, --no-refusals      skip the refusals check
  -h, --help             display this help and exit
  -I, --import=DIR+      add directory DIR to import path
  -m, --model=MODEL      explore model MODEL
      --no-interface-livelock
                         skip interface livelock check at EOT
  -S, --no-strict        do not use strict matching of trail
  -q, --queue-size=SIZE  use queue size=SIZE for simulation [~a]
      --queue-size-defer=SIZE
                         use queue size=SIZE for simulation [~a]
      --queue-size-external=SIZE
                         use queue size=SIZE for simulation [~a]
  -s, --strict           use strict matching of trail
  -t, --trail=TRAIL      use named event trail TRAIL
  -T, --trail-file=FILE  read named event trail from FILE

View:
  ~a
" %queue-size %queue-size-defer %queue-size-external
(get-url "trace"))
        (exit (or (and usage? EXIT_OTHER_FAILURE) EXIT_SUCCESS))))
    options))

(define* (handle-simulate-result data #:key debug? verbose?)
  (let* ((stdout (assoc-ref data "stdout"))
         (labels (trace:get-info stdout 'labels))
         (eligible (trace:get-info stdout 'eligible))
         (trail (trace:get-trail stdout))
         (stderr (assoc-ref data "stderr"))
         (status (assoc-ref data "status")))
    (when stderr
      (display stderr (current-error-port)))
    (when (or debug? verbose?)
      (format #t "trail: ~a\n" trail)
      (format #t "labels: ~a\n" (string-join labels ","))
      (format #t "eligible: ~a\n" (string-join eligible ",")))
    status))

(define (main args)
  (daemon-config-set!)
  (let* ((options (parse-opts args))
         (debug? (ide:command-line:get 'debug))
         (verbose? (ide:command-line:get 'verbose))
         (files (option-ref options '() #f))
         (file-name (car files))
         (imports (multi-opt options 'import))
         (input (piped-file->stream file-name #:debug? debug? #:imports imports))
         (model (catch 'quit
                  (lambda _
                    (let ((ast (call-with-handle-exceptions
                                (cute string->ast input)
                                #:backtrace? debug?
                                #:file-name file-name)))
                      ;; Only use --model option after parsing succeeded
                      (or (option-ref options 'model #f)
                          (and=> (ast:get-model ast) ast:dotted-name))))
                  (lambda _
                    (exit EXIT_OTHER_FAILURE))))
         (result EXIT_SUCCESS))

    (define (simulate-handler message)
      (let ((message (parse-message message)))
        (cond ((simulate? message)
               (let* ((data (simulate-data message))
                      (status (handle-simulate-result data #:debug? debug? #:verbose? verbose?)))
                 (unless (zero? status)
                   (set! result EXIT_FAILURE))
                 (when (isatty? (current-error-port))
                   (format (current-error-port) "See result: ~a\n" (get-url "trace")))))
              (else
               (format (current-error-port)
                       "ide simulate: error: received: ~s\n"
                       message)
               (set! result EXIT_FAILURE)))
        message))

    (unless model
      (format (current-error-port) "~a: No dezyne model found\n" file-name)
      (exit EXIT_OTHER_FAILURE))

    (daemon:start #:debug? debug? #:port %ide-port)
    (let* ((trail-file (option-ref options 'trail-file #f))
           (trail (and trail-file (with-input-from-file trail-file read-string)))
           (trail (option-ref options 'trail trail))
           (trail (or trail
                      (and (not (isatty? (current-input-port)))
                           (input-port? (current-input-port))
                           (read-string (current-input-port)))
                      ""))
           (trail trail-model (string->trail+model trail))
           (model (or model trail-model))
           (trail (string-join trail ","))
           (trail (if (mingw?) (string-append trail ",") trail))
           (no-compliance? (option-ref options 'no-compliance #f))
           (no-deadlock? (option-ref options 'no-deadlock #f))
           (no-interface-livelock?
            (option-ref options 'no-interface-livelock #f))
           (no-refusals? (option-ref options 'no-refusals #f))
           (no-queue-full? (option-ref options 'no-queue-full #f))
           (no-strict? (option-ref options 'no-strict #f))
           (queue-size-check (command-line:get-number 'queue-size))
           (queue-size (option-ref options 'queue-size #f))
           (queue-size-defer-check (command-line:get-number 'queue-size-defer))
           (queue-size-defer (option-ref options 'queue-size #f))
           (queue-size-external-check (command-line:get-number
                                       'queue-size-external))
           (queue-size-external (option-ref options 'queue-size-external #f))
           (command (make-command
                     "simulate"
                     `("dzn" "simulate"
                       ,@(if model `("--model" ,model) '())
                       ,@(if no-strict? '() `("--strict"))
                       ,@(if no-compliance? `("--no-compliance") `())
                       ,@(if no-deadlock? `("--no-deadlock") `())
                       ,@(if no-interface-livelock? `("--no-interface-livelock")
                             `())
                       ,@(if no-queue-full? `("--no-queue-full") `())
                       ,@(if no-refusals? `("--no-refusals") `())
                       ,@(if (not queue-size) '()
                             `("--queue-size" ,queue-size))
                       ,@(if (not queue-size-defer) '()
                             `("--queue-size-defer" ,queue-size-defer))
                       ,@(if (not queue-size-external) '()
                             `("--queue-size-external" ,queue-size-external))
                       "--locations"
                       "--state"
                       "--trail" ,trail
                       "-")
                     #:input input
                     #:model model
                     #:no-compliance? no-compliance?
                     #:no-deadlock? no-deadlock?
                     #:no-interface-livelock? no-interface-livelock?
                     #:no-queue-full? no-queue-full?
                     #:no-refusals? no-refusals?
                     #:no-strict? no-strict?
                     #:queue-size queue-size
                     #:queue-size-defer queue-size-defer
                     #:queue-size-external queue-size-external
                     #:trail trail))
           (commander (make-commander #:debug? debug? #:verbose? verbose?))
           (result-handler
            (compose (make-finalizer commander #:debug? debug? #:verbose? verbose?)
                     simulate-handler))
           (result
            (+ (run-command commander command
                            #:result-handler result-handler
                            #:ide-port %ide-port #:debug? debug? #:verbose? verbose?)
               result)))
      (exit result))))

==> verum-dezyne-2.18.2-x86_64-windows-debug.zip: dezyne-2.18.2/gnu/share/guile/site/3.0/ide/commands/simulate.scm <==
;;; Verum-Dezyne --- An IDE for Dezyne
;;;
;;; Copyright © 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
;;; Copyright © 2020 Rutger van Beusekom <rutger.van.beusekom@verum.com>
;;; Copyright © 2024 Paul Hoogendijk <paul.hoogendijk@verum.com>
;;;
;;; This file is part of Verum-Dezyne.
;;;
;;; Verum-Dezyne is property of Verum Software Tools BV <support@verum.com>.
;;; All rights reserved.

;;
;;; Commentary:
;;;
;;; Code:

(define-module (ide commands simulate)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-26)
  #:use-module (srfi srfi-71)
  #:use-module (ice-9 getopt-long)
  #:use-module (ice-9 match)
  #:use-module (ice-9 rdelim)
  #:use-module (oop goops)
  #:use-module (dzn ast)
  #:use-module (dzn ast goops)
  #:use-module (dzn parse)
  #:use-module (ide shell-util)
  #:use-module (ide commander)
  #:use-module (ide commands daemon)
  #:use-module (ide config)
  #:use-module (ide daemon-config)
  #:use-module (ide pipe)
  #:use-module (ide util)
  #:use-module (ide command-line)
  #:export (sequence->error
            handle-simulate-result))

(define (parse-opts args)
  (let* ((option-spec
          '((help (single-char #\h))
            (import (single-char #\I) (value #t))
            (model (single-char #\m) (value #t))
            (no-compliance (single-char #\C))
            (no-deadlock (single-char #\D))
            (no-interface-livelock)
            (no-queue-full (single-char #\Q))
            (no-refusals (single-char #\R))
            (no-strict (single-char #\S))
            (queue-size (single-char #\q) (value #t))
            (queue-size-defer (value #t))
            (queue-size-external (value #t))
            (strict (single-char #\s))
            (trail (single-char #\t) (value #t))
            (trail-file (single-char #\T) (value #t))))
	 (options (getopt-long args option-spec))
	 (help? (option-ref options 'help #f))
	 (files (option-ref options '() '()))
         (usage? (and (not help?) (null? files))))
    (when (or help? usage?)
      (let ((port (if usage? (current-error-port) (current-output-port))))
        (format port "\
Usage: ide simulate [OPTION]... [FILE]...
Run simulate and make a trace view available.

  -C, --no-compliance    skip the compliance check
  -D, --no-deadlock      skip the deadlock check
  -Q, --no-queue-full    skip the external queue-full check
  -R, --no-refusals      skip the refusals check
  -h, --help             display this help and exit
  -I, --import=DIR+      add directory DIR to import path
  -m, --model=MODEL      explore model MODEL
      --no-interface-livelock
                         skip interface livelock check at EOT
  -S, --no-strict        do not use strict matching of trail
  -q, --queue-size=SIZE  use queue size=SIZE for simulation [~a]
      --queue-size-defer=SIZE
                         use queue size=SIZE for simulation [~a]
      --queue-size-external=SIZE
                         use queue size=SIZE for simulation [~a]
  -s, --strict           use strict matching of trail
  -t, --trail=TRAIL      use named event trail TRAIL
  -T, --trail-file=FILE  read named event trail from FILE

View:
  ~a
" %queue-size %queue-size-defer %queue-size-external
(get-url "trace"))
        (exit (or (and usage? EXIT_OTHER_FAILURE) EXIT_SUCCESS))))
    options))

(define* (handle-simulate-result data #:key debug? verbose?)
  (let* ((stdout (assoc-ref data "stdout"))
         (labels (trace:get-info stdout 'labels))
         (eligible (trace:get-info stdout 'eligible))
         (trail (trace:get-trail stdout))
         (stderr (assoc-ref data "stderr"))
         (status (assoc-ref data "status")))
    (when stderr
      (display stderr (current-error-port)))
    (when (or debug? verbose?)
      (format #t "trail: ~a\n" trail)
      (format #t "labels: ~a\n" (string-join labels ","))
      (format #t "eligible: ~a\n" (string-join eligible ",")))
    status))

(define (main args)
  (daemon-config-set!)
  (let* ((options (parse-opts args))
         (debug? (ide:command-line:get 'debug))
         (verbose? (ide:command-line:get 'verbose))
         (files (option-ref options '() #f))
         (file-name (car files))
         (imports (multi-opt options 'import))
         (input (piped-file->stream file-name #:debug? debug? #:imports imports))
         (model (catch 'quit
                  (lambda _
                    (let ((ast (call-with-handle-exceptions
                                (cute string->ast input)
                                #:backtrace? debug?
                                #:file-name file-name)))
                      ;; Only use --model option after parsing succeeded
                      (or (option-ref options 'model #f)
                          (and=> (ast:get-model ast) ast:dotted-name))))
                  (lambda _
                    (exit EXIT_OTHER_FAILURE))))
         (result EXIT_SUCCESS))

    (define (simulate-handler message)
      (let ((message (parse-message message)))
        (cond ((simulate? message)
               (let* ((data (simulate-data message))
                      (status (handle-simulate-result data #:debug? debug? #:verbose? verbose?)))
                 (unless (zero? status)
                   (set! result EXIT_FAILURE))
                 (when (isatty? (current-error-port))
                   (format (current-error-port) "See result: ~a\n" (get-url "trace")))))
              (else
               (format (current-error-port)
                       "ide simulate: error: received: ~s\n"
                       message)
               (set! result EXIT_FAILURE)))
        message))

    (unless model
      (format (current-error-port) "~a: No dezyne model found\n" file-name)
      (exit EXIT_OTHER_FAILURE))

    (daemon:start #:debug? debug? #:port %ide-port)
    (let* ((trail-file (option-ref options 'trail-file #f))
           (trail (and trail-file (with-input-from-file trail-file read-string)))
           (trail (option-ref options 'trail trail))
           (trail (or trail
                      (and (not (isatty? (current-input-port)))
                           (input-port? (current-input-port))
                           (read-string (current-input-port)))
                      ""))
           (trail trail-model (string->trail+model trail))
           (model (or model trail-model))
           (trail (string-join trail ","))
           (trail (if (mingw?) (string-append trail ",") trail))
           (no-compliance? (option-ref options 'no-compliance #f))
           (no-deadlock? (option-ref options 'no-deadlock #f))
           (no-interface-livelock?
            (option-ref options 'no-interface-livelock #f))
           (no-refusals? (option-ref options 'no-refusals #f))
           (no-queue-full? (option-ref options 'no-queue-full #f))
           (no-strict? (option-ref options 'no-strict #f))
           (queue-size-check (command-line:get-number 'queue-size))
           (queue-size (option-ref options 'queue-size #f))
           (queue-size-defer-check (command-line:get-number 'queue-size-defer))
           (queue-size-defer (option-ref options 'queue-size-defer #f))
           (queue-size-external-check (command-line:get-number
                                       'queue-size-external))
           (queue-size-external (option-ref options 'queue-size-external #f))
           (command (make-command
                     "simulate"
                     `("dzn" "simulate"
                       ,@(if model `("--model" ,model) '())
                       ,@(if no-strict? '() `("--strict"))
                       ,@(if no-compliance? `("--no-compliance") `())
                       ,@(if no-deadlock? `("--no-deadlock") `())
                       ,@(if no-interface-livelock? `("--no-interface-livelock")
                             `())
                       ,@(if no-queue-full? `("--no-queue-full") `())
                       ,@(if no-refusals? `("--no-refusals") `())
                       ,@(if (not queue-size) '()
                             `("--queue-size" ,queue-size))
                       ,@(if (not queue-size-defer) '()
                             `("--queue-size-defer" ,queue-size-defer))
                       ,@(if (not queue-size-external) '()
                             `("--queue-size-external" ,queue-size-external))
                       "--locations"
                       "--state"
                       "--trail" ,trail
                       "-")
                     #:input input
                     #:model model
                     #:no-compliance? no-compliance?
                     #:no-deadlock? no-deadlock?
                     #:no-interface-livelock? no-interface-livelock?
                     #:no-queue-full? no-queue-full?
                     #:no-refusals? no-refusals?
                     #:no-strict? no-strict?
                     #:queue-size queue-size
                     #:queue-size-defer queue-size-defer
                     #:queue-size-external queue-size-external
                     #:trail trail))
           (commander (make-commander #:debug? debug? #:verbose? verbose?))
           (result-handler
            (compose (make-finalizer commander #:debug? debug? #:verbose? verbose?)
                     simulate-handler))
           (result
            (+ (run-command commander command
                            #:result-handler result-handler
                            #:ide-port %ide-port #:debug? debug? #:verbose? verbose?)
               result)))
      (exit result))))
